"use strict";
/**
 * Copyright 2022 Project CHIP Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
const SRC = '../../';
const Const = require(SRC + 'Const');
const logger = require(SRC + 'util/Logger');
const Random = require(SRC + 'util/Random');
const { TlvReader } = require(SRC + 'tlv/TlvReader');
const { TlvWriter } = require(SRC + 'tlv/TlvWriter');
const CommandPath = require(SRC + "protocols/interaction_model/CommandPath");
const InteractionModel = require(SRC + "protocols/interaction_model/InteractionModel");
/**
 * ClusterNetworkCommissioningServer
 */
class ClusterNetworkCommissioningServer {
    // ============================================
    //             ATTRIBUTE TEMPLATES
    // ============================================
    constructor(imManager) {
        this._imManager = imManager;
        this._dataVersion = Random.getRandomUint32();
        this._attributes = {
            0: {
                "name": "MaxNetworks",
                "type": "int8u",
                "nullable": false,
                "nosubscribe": true,
                "readonly": true,
            },
            1: {
                "name": "Networks",
                "type": "array",
                "nullable": false,
                "nosubscribe": true,
                "readonly": true,
            },
            2: {
                "name": "ScanMaxTimeSeconds",
                "type": "int8u",
                "nullable": false,
                "nosubscribe": true,
                "readonly": true,
            },
            3: {
                "name": "ConnectMaxTimeSeconds",
                "type": "int8u",
                "nullable": false,
                "nosubscribe": true,
                "readonly": true,
                "value": 0
            },
            4: {
                "name": "InterfaceEnabled",
                "type": "boolean",
                "nullable": false,
                "nosubscribe": true,
                "readonly": true,
            },
            5: {
                "name": "LastNetworkingStatus",
                "type": "NetworkCommissioningStatus",
                "nullable": true,
                "nosubscribe": true,
                "readonly": true,
            },
            6: {
                "name": "LastNetworkID",
                "type": "octet_string",
                "nullable": true,
                "nosubscribe": true,
                "readonly": true,
            },
            7: {
                "name": "LastConnectErrorValue",
                "type": "int32s",
                "nullable": true,
                "nosubscribe": true,
                "readonly": true,
            },
            65528: {
                "name": "GeneratedCommandList",
                "type": "array",
                "nullable": false,
                "nosubscribe": true,
                "readonly": true,
            },
            65529: {
                "name": "AcceptedCommandList",
                "type": "array",
                "nullable": false,
                "nosubscribe": true,
                "readonly": true,
            },
            65531: {
                "name": "AttributeList",
                "type": "array",
                "nullable": false,
                "nosubscribe": true,
                "readonly": true,
            },
            65532: {
                "name": "FeatureMap",
                "type": "bitmap32",
                "nullable": false,
                "nosubscribe": true,
                "readonly": true,
                "value": 4,
            },
            65533: {
                "name": "ClusterRevision",
                "type": "int16u",
                "nullable": false,
                "nosubscribe": true,
                "readonly": true,
            },
        };
    }
    getAttribute(id) {
        return this._attributes[id];
    }
    onCommand(msg, path) {
        logger.debug(this.constructor.name + ".onCommand " + path.CommandId);
        switch (path.CommandId) {
            case ClusterGeneralCommissioning.Command.ScanNetworks:
                this.onScanNetworks(msg, path);
                break;
            case ClusterGeneralCommissioning.Command.ScanNetworkResponse:
                this.onScanNetworkResponse(msg, path);
                break;
            case ClusterGeneralCommissioning.Command.AddOrUpdateWiFiNetwork:
                this.onAddOrUpdateWiFiNetwork(msg, path);
                break;
        }
    }
    onScanNetworks(msg, commandPath) {
        logger.debug(this.constructor.name + '.onScanNetworks');
        this._imManager.sendInvokeCommandResponseStatus(exchange, commandPath, 0);
    }
    onScanNetworkResponse(msg, commandPath) {
        logger.debug(this.constructor.name + '.onScanNetworkResponse');
        this._imManager.sendInvokeCommandResponseStatus(exchange, commandPath, 0);
    }
    onAddOrUpdateWiFiNetwork(msg, commandPath) {
        logger.debug(this.constructor.name + '.onAddOrUpdateWiFiNetwork');
        this._imManager.sendInvokeCommandResponseStatus(exchange, commandPath, 0);
    }
}
ClusterNetworkCommissioningServer.CLUSTER_NAME = 'NETWORK_COMMISSIONING_CLUSTER';
ClusterNetworkCommissioningServer.CLUSTER_ID = 0x0031;
ClusterNetworkCommissioningServer.Command = {
    ScanNetworks: 0,
    ScanNetworksResponse: 1,
    AddOrUpdateWiFiNetwork: 2,
    AddOrUpdateThreadNetwork: 3,
    RemoveNetwork: 4,
    NetworkConfigResponse: 5,
    ConnectNetwork: 6,
    ConnectNetworkResponse: 7,
    ReorderNetwork: 8,
};
ClusterNetworkCommissioningServer.Attribute = {
    MaxNetworks: 0,
    Networks: 1,
    ScanMaxTimeSeconds: 2,
    ConnectMaxTimeSeconds: 3,
    InterfaceEnabled: 4,
    LastNetworkingStatus: 5,
    LastNetworkID: 6,
    LastConnectErrorValue: 7,
    GeneratedCommandList: 65528,
    AcceptedCommandList: 65529,
    AttributeList: 65531,
    FeatureMap: 65532,
    ClusterRevision: 65533,
};
// ============================================
//              COMMAND TEMPLATES
// ============================================
ClusterNetworkCommissioningServer.TemplateCommandScanNetworks = function (params) {
    return [
        { 'tag': 0, 'type': 'chip::ByteSpan', 'value': params.Ssid },
        { 'tag': 1, 'type': 'uint64_t', 'value': params.Breadcrumb },
    ];
};
ClusterNetworkCommissioningServer.TemplateCommandScanNetworksResponse = function (params) {
    return [
        { 'tag': 0, 'type': 'uint8_t', 'value': params.NetworkingStatus },
        { 'tag': 1, 'type': 'chip::CharSpan', 'value': params.DebugText },
        { 'tag': 2, 'type': 'array', 'value': [] },
        { 'tag': 3, 'type': 'array', 'value': [] }, // params.ThreadScanResults
    ];
};
ClusterNetworkCommissioningServer.TemplateCommandAddOrUpdateWiFiNetwork = function (params) {
    return [
        { 'tag': 0, 'type': 'chip::ByteSpan', 'value': params.Ssid },
        { 'tag': 1, 'type': 'chip::ByteSpan', 'value': params.Credentials },
        { 'tag': 2, 'type': 'uint64_t', 'value': params.Breadcrumb },
    ];
};
ClusterNetworkCommissioningServer.TemplateCommandAddOrUpdateThreadNetwork = function (params) {
    return [
        { 'tag': 0, 'type': 'chip::ByteSpan', 'value': params.OperationalDataset },
        { 'tag': 1, 'type': 'uint64_t', 'value': params.Breadcrumb },
    ];
};
ClusterNetworkCommissioningServer.TemplateCommandRemoveNetwork = function (params) {
    return [
        { 'tag': 0, 'type': 'chip::ByteSpan', 'value': params.NetworkID },
        { 'tag': 1, 'type': 'uint64_t', 'value': params.Breadcrumb },
    ];
};
ClusterNetworkCommissioningServer.TemplateCommandNetworkConfigResponse = function (params) {
    return [
        { 'tag': 0, 'type': 'uint8_t', 'value': params.NetworkingStatus },
        { 'tag': 1, 'type': 'chip::CharSpan', 'value': params.DebugText },
        { 'tag': 2, 'type': 'uint8_t', 'value': params.NetworkIndex },
    ];
};
ClusterNetworkCommissioningServer.TemplateCommandConnectNetwork = function (params) {
    return [
        { 'tag': 0, 'type': 'chip::ByteSpan', 'value': params.NetworkID },
        { 'tag': 1, 'type': 'uint64_t', 'value': params.Breadcrumb },
    ];
};
ClusterNetworkCommissioningServer.TemplateCommandConnectNetworkResponse = function (params) {
    return [
        { 'tag': 0, 'type': 'uint8_t', 'value': params.NetworkingStatus },
        { 'tag': 1, 'type': 'chip::CharSpan', 'value': params.DebugText },
        { 'tag': 2, 'type': 'int32_t', 'value': params.ErrorValue },
    ];
};
ClusterNetworkCommissioningServer.TemplateCommandReorderNetwork = function (params) {
    return [
        { 'tag': 0, 'type': 'chip::ByteSpan', 'value': params.NetworkID },
        { 'tag': 1, 'type': 'uint8_t', 'value': params.NetworkIndex },
        { 'tag': 2, 'type': 'uint64_t', 'value': params.Breadcrumb },
    ];
};
module.exports = ClusterNetworkCommissioningServer;
//# sourceMappingURL=ClusterNetworkCommissioningServer.js.map