"use strict";
/**
 * @license
 * Copyright (c) 2022 Project CHIP Authors. All rights reserved.
 * SPDX-License-Identifier: Apache-2.0
 */
const SRC = "../";
const logger = require(SRC + "util/Logger");
const Crypto = require(SRC + "crypto/Crypto");
const CommandPath = require(SRC + "protocols/interaction_model/CommandPath");
const ClusterGeneralCommissioning = require(SRC + "clusters/commissioning/ClusterGeneralCommissioning");
class Commissioner {
    /**
     * Create a new commissioner instance.
     *
     * @param {Node} node
     */
    constructor(imManager, node, params) {
        this._imManager = imManager;
        this._node = node;
        //this._deviceDelegate = params.deviceDelegate
        //this._pairingDelegate = params.pairingDelegate
        //this._pairingDelegate.on('pairingComplete', this._onPairingComplete)
        this._onPairingError = null;
        this._onOperationalCredentialsProvisioningError = null;
        this._onOperationalCredentialsProvisioningComplete = null;
    }
    /**
     * Returns next stage of the state machine give the current one.
     * @param {StageEnum} stage
     * @returns  {StageEnum} nextStage
     */
    getNextStage(stage) {
        switch (stage) {
            // ================================
            //     CORE COMMISSIONING STAGES
            // ================================
            case Commissioner.StageEnum.kSecurePairing:
                return Commissioner.StageEnum.kReadCommissioningInfo;
            case Commissioner.StageEnum.kReadCommissioningInfo:
                if (this._commissioningInfo.breadcrumbs > 0) {
                    // If the breadcrumb is 0, the failsafe was disarmed.
                    // We failed on network setup or later, the node failsafe 
                    // has not been re-armed and the breadcrumb has not been reset.
                    // Per the spec, we restart from after adding the NOC.
                    return Commissioner.StageEnum.kNeedsNetworkCreds;
                }
                else {
                    return Commissioner.StageEnum.kArmFailsafe;
                }
            case Commissioner.StageEnum.kArmFailsafe:
                return Commissioner.StageEnum.kConfigRegulatory;
            case Commissioner.StageEnum.kNeedsNetworkCreds:
                return Commissioner.StageEnum.kConfigRegulatory;
            case Commissioner.StageEnum.kConfigRegulatory:
                return Commissioner.StageEnum.kSendPaiCertificateRequest;
            case Commissioner.StageEnum.kSendPaiCertificateRequest:
                return Commissioner.StageEnum.kSendDacCertificateRequest;
            case Commissioner.StageEnum.kSendDacCertificateRequest:
                return Commissioner.StageEnum.kSendAttestationRequest;
            case Commissioner.StageEnum.kSendAttestationRequest:
                return Commissioner.StageEnum.kAttestationVerification;
            case Commissioner.StageEnum.kAttestationVerification:
                return Commissioner.StageEnum.kSendOpCertSigningRequest;
            case Commissioner.StageEnum.kSendOpCertSigningRequest:
                return Commissioner.StageEnum.kValidateCsr;
            case Commissioner.StageEnum.kValidateCsr:
                return Commissioner.StageEnum.kGenerateNocChain;
            case Commissioner.StageEnum.kGenerateNocChain:
                return Commissioner.StageEnum.kSendTrustedRootCert;
            case Commissioner.StageEnum.kSendTrustedRootCert:
                return Commissioner.StageEnum.kSendNoc;
            case Commissioner.StageEnum.kSendNoc:
                return Commissioner.StageEnum.kFindOperational;
            case Commissioner.StageEnum.kFindOperational:
                return Commissioner.StageEnum.kSendComplete;
            case Commissioner.StageEnum.kSendComplete:
                return Commissioner.StageEnum.kCleanup;
            // ================================
            //   NETWORK COMMISSIONING STAGES
            // ================================
            case Commissioner.StageEnum.kNeedsNetworkCreds: break;
            case Commissioner.StageEnum.kScanNetworks:
                return Commissioner.StageEnum.kNeedsNetworkCreds;
            case Commissioner.StageEnum.kWiFiNetworkSetup:
                if (this._commissioningInfo.setupThread) {
                    return Commissioner.StageEnum.kThreadNetworkSetup;
                }
                return Commissioner.StageEnum.kWiFiNetworkEnable;
            case Commissioner.StageEnum.kThreadNetworkSetup:
                if (this._commissioningInfo.setupWifi) {
                    return Commissioner.StageEnum.kWifiNetworkSetup;
                }
                return Commissioner.StageEnum.kThreadNetworkEnable;
            case Commissioner.StageEnum.kWiFiNetworkEnable:
                if (this._commissioningInfo.setupThread) {
                    return Commissioner.StageEnum.kThreadNetworkEnable;
                }
                return Commissioner.StageEnum.kFindOperational;
            case Commissioner.StageEnum.kThreadNetworkEnable:
                if (this._commissioningInfo.setupWifi) {
                    return Commissioner.StageEnum.kWiFiNetworkEnable;
                }
                return Commissioner.StageEnum.kFindOperational;
            // None of these have actual stage, so return error.
            case Commissioner.StageEnum.kCleanup:
            case Commissioner.StageEnum.kError:
            default:
                return Commissioner.StageEnum.kError;
        }
    }
    stageSecurePairing() {
        var self = this;
        const ipHost = this._device.ipHost;
        const ipPort = this._device.ipPort;
        const pincode = this._pincode;
        // TODO: Simplify initiating exchange on the unsecurfed cliet session.
        var exchange = this._node.ExchangeManager.newExchange();
        exchange.Session.Transport = this._node.Client;
        exchange.Session.PeerInfo = {
            port: ipPort,
            address: ipHost,
        };
        var pase = this._node.SessionProtocol.newPase(exchange);
        pase.on('onPairingComplete', () => { self.onPairingComplete(); });
        pase.on('onPairingError', () => { self.onPairingError(); });
        pase.startPase(ipHost, ipPort, pincode);
    }
    onPairingComplete(session) {
        // TODO: Send IM command via given secure session.
        logger.info("Commissioner: PASE done");
        this._session = session;
        this.stageArmFailSafe();
        this._exchange = this._node.ExchangeManager.newExchange(session);
        //this._exchange.on('onMessageReceived', this.onMessageReceived)
        //this._stage = this.getNextStage(this._stage)
        //this.performStage(this._stage)
    }
    onPairingError(session) {
        logger.error("Commissioner: PASE error");
    }
    stageReadCommissioningInfo() {
    }
    onCommissioningInfoReport() {
    }
    stageArmFailSafe() {
        logger.debug(this.constructor.name + ".stageArmFailSafeResponse");
        if (this._state != Commissioner.StageEnum.kSecurePairing) {
            logger.error("Commissioner: Unexpected state");
            return;
        }
        this._state = Commissioner.StageEnum.kArmFailsafe;
        var params = {
            Breadcrumb: Crypto.RandomU32(),
            ExpiryLengthSeconds: 10000,
        };
        var jsonObj = ClusterGeneralCommissioning.TemplateCommandArmFailSafe(params);
        var commandDataJson = JSON.stringify(jsonObj, null, 2);
        logger.trace(commandDataJson, null, 2);
        var commandPath = new CommandPath();
        commandPath.EndpointId = 0;
        commandPath.ClusterId = ClusterGeneralCommissioning.CLUSTER_ID;
        commandPath.CommandId = ClusterGeneralCommissioning.Command.ArmFailSafe;
        this._imManager.sendInvokeCommandRequestData(exchange, commandPath, jsonObj);
    }
    onArmFailSafe() {
        if (this._state != Commissioner.StageEnum.kArmFailsafe) {
            logger.error("Commissioner: Unexpected state");
            return;
        }
    }
    stageConfigRegulatory() {
    }
    onConfigRegulatory() {
    }
    stageSendPaiCertificateRequest() {
    }
    onPaiCertificateResponse() {
    }
    stageSendDacCertificateRequest() {
    }
    onDacCertificateResponse() {
    }
    stageSendAttestationRequest() {
    }
    onAttestationResponse() {
    }
    stageAttestationVerification() {
    }
    stageSendOpCertSigningRequest() {
    }
    onOpCertSigningResponse() {
    }
    stageValidateCsr() {
    }
    stageGenerateNocChain() {
    }
    stageSendTrustedRootCert() {
    }
    stageSendNoc() {
    }
    stageFindOperational() {
    }
    stageSendComplete() {
    }
    /**
     * Dispatch work for the current stage of the state machine and await the
     * transition event that will trigger the next stage.
     *
     * @param {StageEnum} stage
     */
    performStage(stage) {
        switch (stage) {
            // ================================
            //     CORE COMMISSIONING STAGES
            // ================================
            case Commissioner.StageEnum.kSecurePairing: return this.stageSecurePairing();
            case Commissioner.StageEnum.kReadCommissioningInfo: return this.stageReadCommissioningInfo();
            case Commissioner.StageEnum.kArmFailsafe: return this.stageArmFailSafe();
            case Commissioner.StageEnum.kNeedsNetworkCreds: return this.stageNeedsNetworkCreds();
            case Commissioner.StageEnum.kConfigRegulatory: return this.stageConfigRegulatory();
            case Commissioner.StageEnum.kSendPaiCertificateRequest: return this.stageSendPaiCertificateRequest();
            case Commissioner.StageEnum.kSendDacCertificateRequest: return this.stageSendDacCertificateRequest();
            case Commissioner.StageEnum.kSendAttestationRequest: return this.stageSendAttestationRequest();
            case Commissioner.StageEnum.kAttestationVerification: return this.stageAttestationVerification();
            case Commissioner.StageEnum.kSendOpCertSigningRequest: return this.stageSendOpCertSigningRequest();
            case Commissioner.StageEnum.kValidateCsr: return this.stageValidateCsr();
            case Commissioner.StageEnum.kGenerateNocChain: return this.stageGenerateNocChain();
            case Commissioner.StageEnum.kSendTrustedRootCert: return this.stageSendTrustedRootCert();
            case Commissioner.StageEnum.kSendNoc: return this.stageSendNoc();
            case Commissioner.StageEnum.kFindOperational: return this.stageFindOperational();
            case Commissioner.StageEnum.kSendComplete: return this.stageSendComplete();
            // ================================
            //   NETWORK COMMISSIONING STAGES
            // ================================
            case Commissioner.StageEnum.kNeedsNetworkCreds: return this.stageNeedsNetworkCreds();
            case Commissioner.StageEnum.kScanNetworks: return this.stageScanNetworks();
            case Commissioner.StageEnum.kWiFiNetworkSetup: return this.stageWiFiNetworkSetup();
            case Commissioner.StageEnum.kThreadNetworkSetup: return this.stageThreadNetworkSetup();
            case Commissioner.StageEnum.kWiFiNetworkEnable: return this.stageWiFiNetworkEnable();
            case Commissioner.StageEnum.kThreadNetworkEnable: return this.stageThreadNetworkEnable();
            // None of these have actual stage, so return error.
            case Commissioner.StageEnum.kCleanup:
            case Commissioner.StageEnum.kError:
            default:
                return;
        }
    }
    /**
     * Commission the given device onto the given fabric.
     * //@param {*} device
     * @param {*} fabric
     */
    pair(device, pincode, nodeid, fabric) {
        this._device = device;
        this._pincode = pincode;
        this._device._nodeid = nodeid;
        this._device._fabric = fabric;
        this._stage = Commissioner.StageEnum.kSecurePairing;
        this.performStage(this._stage);
    }
}
Commissioner.StageEnum = {
    kError: 0,
    kSecurePairing: 1,
    kReadCommissioningInfo: 2,
    kArmFailsafe: 3,
    kConfigRegulatory: 4,
    kSendPaiCertificateRequest: 5,
    kSendDacCertificateRequest: 6,
    kSendAttestationRequest: 7,
    kAttestationVerification: 8,
    kSendOpCertSigningRequest: 9,
    kValidateCsr: 10,
    kGenerateNocChain: 11,
    kSendTrustedRootCert: 12,
    kSendNoc: 13,
    kWiFiNetworkSetup: 14,
    kThreadNetworkSetup: 15,
    kWiFiNetworkEnable: 16,
    kThreadNetworkEnable: 17,
    kFindOperational: 18,
    kSendComplete: !9,
    kCleanup: 20,
    kScanNetworks: 21,
    kNeedsNetworkCreds: 22, ///< Setup appropriate network commissioning based on kReadCommissioningInfo response.
};
module.exports = Commissioner;
//# sourceMappingURL=Commissioner.js.map