"use strict";
/**
 * Copyright 2022 Project CHIP Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
class ChipCert {
}
// As per specifications (6.3.5. Node Operational Credentials Certificates)
ChipCert.kMaxCHIPCertLength = 400;
ChipCert.kMaxDERCertLength = 600;
/// Data Element Tags for the CHIP Certificate
ChipCert.TlvTagStructure = {
    // ---- Context-specific Tags for ChipCertificate Structure ----
    SerialNumber: 1,
    SignatureAlgorithm: 2,
    Issuer: 3,
    NotBefore: 4,
    NotAfter: 5,
    Subject: 6,
    PublicKeyAlgorithm: 7,
    EllipticCurveId: 8,
    PublicKey: 9,
    Extensions: 10,
    Signature: 11, ///< [ byte string ] The ECDSA signature for the certificate.
};
ChipCert.TlvTagExtensions = {
    // ---- Context-specific Tags for certificate extensions ----
    BasicConstraints: 1,
    KeyUsage: 2,
    ExtendedKeyUsage: 3,
    SubjectKeyId: 4,
    AuthorityKeyId: 5,
    FutureExtension: 6, ///< [ byte string ] Arbitrary extension. DER encoded SEQUENCE as in X.509 form.
};
ChipCert.TlvTagDn = {
    // Standard and Matter-specific DN attributes.
    // Of these, all are encoded as UTF8String except domain-component,
    // which is encoded as IA5String in X.509 form.
    CommonName: 1,
    Surname: 2,
    SerialNumber: 3,
    CountryName: 4,
    LocalityName: 5,
    StateOrProvinceName: 6,
    OrgName: 7,
    OrgUnitName: 8,
    Title: 9,
    Name: 10,
    GivenName: 11,
    Initials: 12,
    GenQualifier: 13,
    DnQualifier: 14,
    Pseudonym: 15,
    DomainComponent: 16,
    MatterNodeId: 17,
    MatterFirmwareSigningId: 18,
    MatterIcacId: 19,
    MatterRcacId: 20,
    MatterFabricId: 21,
    MatterNocCat: 22,
    // Standard DN attributes when encoded as PrintableString in X.509 form
    // NOTE: The tags for these SHALL be the base tags + 0x80.
    CommonNamePs: 129,
    SurnamePs: 130,
    SerialNumPs: 131,
    CountryNamePs: 132,
    LocalityNamePs: 133,
    StateOrProvinceNamePs: 134,
    OrgNamePs: 135,
    OrgUnitNamePs: 136,
    TitlePs: 137,
    NamePs: 138,
    GivenNamePs: 139,
    InitialsPs: 140,
    GenQualifierPs: 141,
    DnQualifierPs: 143,
    PseudonymOs: 144,
};
ChipCert.TlvTagBasicConstraints = {
    // ---- Context-specific Tags for BasicConstraints Structure ----
    kTag_BasicConstraints_IsCA: 1,
    kTag_BasicConstraints_PathLenConstraint: 2, ///< [ unsigned int ] Maximum number of subordinate intermediate certificates.
};
/** Identifies the purpose or application of certificate
 *
 * A certificate type is a label that describes a certificate's purpose or application.
 * Certificate types are not carried as attributes of the corresponding certificates, but
 * rather are derived from the certificate's structure and/or the context in which it is used.
 * The certificate type enumeration includes a set of pre-defined values describing commonly
 * used certificate applications.
 *
 * Certificate types are primarily used in the implementation of access control policies,
 * where access to application features is influenced by the type of certificate presented
 * by a requester.
 *
 * @note Cert type is an API data type only; it should never be sent over-the-wire.
 */
ChipCert.CertType = {
    kCertType_NotSpecified: 0x00,
    kCertType_Root: 0x01,
    kCertType_ICA: 0x02,
    kCertType_Node: 0x03,
    kCertType_FirmwareSigning: 0x04, /**< A CHIP firmware signing certificate. Note that CHIP doesn't
                                       specify how firmware images are signed and implementation of
                                       firmware image signing is manufacturer-specific. The CHIP
                                       certificate format supports encoding of firmware signing
                                       certificates if chosen by the manufacturer to use them. */
};
/** X.509 Certificate Key Purpose Flags
 *
 *    The flags order must match the enumeration order of corresponding kOID_KeyPurpose values.
 */
ChipCert.KeyPurposeFlags = {
    kServerAuth: 0x01,
    kClientAuth: 0x02,
    kCodeSigning: 0x04,
    kEmailProtection: 0x08,
    kTimeStamping: 0x10,
    kOCSPSigning: 0x20, ///< Extended key usage is OCSP signing.
};
/** X.509 Certificate Key Usage Flags
 */
ChipCert.KeyUsageFlags = {
    kDigitalSignature: 0x0001,
    kNonRepudiation: 0x0002,
    kKeyEncipherment: 0x0004,
    kDataEncipherment: 0x0008,
    kKeyAgreement: 0x0010,
    kKeyCertSign: 0x0020,
    kCRLSign: 0x0040,
    kEncipherOnly: 0x0080,
    kDecipherOnly: 0x0100, ///< Key usage is decipher only.
};
/** CHIP Certificate Flags
 *
 * Contains information about a certificate that has been loaded into a ChipCertificateData object.
 */
ChipCert.CertFlags = {
    kExtPresent_BasicConstraints: 0x0001,
    kExtPresent_KeyUsage: 0x0002,
    kExtPresent_ExtendedKeyUsage: 0x0004,
    kExtPresent_SubjectKeyId: 0x0008,
    kExtPresent_AuthKeyId: 0x0010,
    kExtPresent_FutureIsCritical: 0x0020,
    kPathLenConstraintPresent: 0x0040,
    kIsCA: 0x0080,
    kIsTrustAnchor: 0x0100,
    kTBSHashPresent: 0x0200, ///< Indicates that TBS hash of the certificate was generated and stored.
};
/** CHIP Certificate Decode Flags
 *
 * Contains information specifying how a certificate should be decoded.
 */
ChipCert.CertDecodeFlags = {
    kGenerateTBSHash: 0x01,
    kIsTrustAnchor: 0x02, ///< Indicates that the corresponding certificate is trust anchor.
};
ChipCert.kNullCertTime = 0;
module.exports = ChipCert;
//# sourceMappingURL=ChipCert.js.map