export = Crypto;
/**
 * Crypto
 */
declare class Crypto {
    static CRYPTO_AEAD_MIC_LENGTH_BYTES: number;
    static CRYPTO_AEAD_CCM_NONCE_SIZE: number;
    static CRYPTO_AEAD_CTR_NONCE_SIZE: number;
    static CRYPTO_SYMMETRIC_KEY_LENGTH_BYTES: number;
    /**
     * Crypto_GenerateKeyPair
     * @see 3.5.2. Key generation
     */
    static GenerateKeyPair(): {
        privateKey: Buffer;
        publicKey: Buffer;
    };
    static PublicFromPrivate(privateKey: any): Buffer;
    /**
     * Crypto.Hash computes the cryptographic hash of a message.
     * @see 3.3. Hash function (Hash)
     *
     * @param {Buffer} message
     * @returns {Buffer} hash
     */
    static Hash(message: Buffer): Buffer;
    /**
     * Crypto.Hmac computes the Keyed-Hash Message Authentication Code (HMAC).
     * @see 3.4. Keyed-Hash Message Authentication Code (HMAC)
     *
     * @param {Buffer} key
     * @param {Buffer} message
     * @returns {Buffer} hmac
     */
    static Hmac(key: Buffer, message: Buffer): Buffer;
    /**
     * Crypto.Kdf derives a new key with the given inputs.
     * @see 3.8. Key Derivation Function (KDF)
     *
     * @param {Buffer} inputKey
     * @param {Buffer} salt
     * @param {Buffer} info
     * @param {number} length [bytes]
     * @returns {Buffer} outputKey
     */
    static Kdf(inputKey: Buffer, salt: Buffer, info: Buffer, length: number): Buffer;
    /**
     * Crypto_Sign generates a signature for the given message
     * using the deterministic ECDSA-with-P256 algorithm.
     *
     * @see RFC6979
     * @see 3.5.3. Signature and verification
     *
     * @param {Buffer} message
     * @param {Buffer} privateKey
     *
     * @returns {Buffer} [64 byte]: signature
     */
    static Sign(message: Buffer, privateKey: Buffer): Buffer;
    /**
     * Crypto_Verify validates the given ECDSA signature for a message.
     *
     * @see RFC6979
     * @see 3.5.3. Signature and verification
     *
     * @param {Buffer} message
     * @param {Buffer} signature  [64 bytes]: r || s
     * @param {Buffer} publicKey  [65 bytes]: 04 || x || y
     *
     * @returns {boolean} isValid
     */
    static Verify(message: Buffer, signature: Buffer, publicKey: Buffer): boolean;
    /**
     * AeadEncrypt encrypts the given message using the given key.
     * @see 3.6.1. Generate and encrypt
     *
     * byte[lengthInBytes(P) + CRYPTO_AEAD_MIC_LENGTH_BYTES]
     *   Crypto_AEAD_GenerateEncrypt(
     *           SymmetricKey K,
     *           byte[lengthInBytes(P)] P,
     *           byte[] A,
     *           byte[CRYPTO_AEAD_NONCE_LENGTH_BYTES] N)
     *
     * @param {Buffer} key
     * @param {Buffer} message
     * @param {Buffer} additionalData
     * @param {Buffer} nonce
     * @returns {Buffer} ciphertext
     */
    static AeadEncrypt(key: Buffer, plaintext: any, additionalData: Buffer, nonce: Buffer): Buffer;
    /**
     * AeadDecrypt decrypts the given ciphertext using the given key.
     * @see 3.6.2. Decrypt and verify
     *
     * {boolean success, byte[lengthInBytes(P)] payload}
     *   Crypto_AEAD_DecryptVerify(
     *      SymmetricKey K,
     *      byte[lengthInBytes(P) + CRYPTO_AEAD_MIC_LENGTH_BYTES] C,
     *      byte[] A,
     *      byte[CRYPTO_AEAD_NONCE_LENGTH_BYTES] N)
     */
    static AeadDecrypt(key: any, ciphertext: any, additionalData: any, nonce: any, mic: any): Buffer | null;
    /**
     *
     * @see 3.7.1. Privacy encryption
     * byte[lengthInBytes(M)]
     *   Crypto_Privacy_Encrypt(
     *      SymmetricKey K,
     *      byte[lengthInBytes(M)] M,
     *      byte[CRYPTO_PRIVACY_NONCE_LENGTH_BYTES] N)
     *
     * Crypto_Privacy_Encrypt(K, M, N) :=
     *      byte[lengthInBytes(M)]
     *      AES-CTR-Encrypt(K := K, P := M, N := N)
     */
    static PrivacyEncrypt(symmetricKey: any, plaintext: any, nonce: any): Buffer;
    /**
     * @see 3.7.2. Privacy decryption
     *
     * byte[lengthInBytes(C)]
     *   Crypto_Privacy_Decrypt(
     *      SymmetricKey K,
     *      byte[lengthInBytes(C)] C,
     *      byte[CRYPTO_PRIVACY_NONCE_LENGTH_BYTES] N)
     *
     * Crypto_Privacy_Decrypt(K, C, N) :=
     *      byte[lengthInBytes(C)]
     *      AES-CTR-Decrypt(K := K, C := C, N := N)
     */
    static PrivacyDecrypt(symmetricKey: any, ciphertext: any, nonce: any): Buffer;
    /**
     * Crypto_GenerateCsr
     *
     * @param {*} rawPrivateKey
     * @returns {Buffer} CSR
     */
    static GenerateCsr(rawPrivateKey: any): Buffer;
    /**
     * Generates the CompressedFabricId for this Fabric.
     *
     * CompressedFabricIdentifier =
     * Crypto_KDF(
     *   inputKey := TargetOperationalRootPublicKey,
     *   salt:= TargetOperationalFabricID,
     *   info := CompressedFabricInfo,
     *   len := 64 bits)
     *
     * @param {Buffer} rootPublicKey
     * @param {BigInt} fabricId
     * @returns {Buffer} compressedFabricId
     */
    static GenerateCompressedFabricId(rootPublicKey: Buffer, fabricId: BigInt): Buffer;
    /**
     * Returns a Buffer of random bytes of the given length.
     *
     * bit[len] Crypto_DRBG(int len)
     *
     * @see 3.1 Deterministic Random Bit Generator (DRBG)
     */
    static Random(size: any): Buffer;
    /**
     * Returns a random 16-bit unsigned integer.
     */
    static RandomU16(): number;
    /**
     * Returns a random 32-bit unsigned integer.
     */
    static RandomU32(): number;
    /**
     * Returns a random 64-bit unsigned integer.
     */
    static RandomU64(): bigint;
}
