export = MsgCodec;
/**
 * A class for encode and decode of Matter Messages.
 */
declare class MsgCodec {
    static DEFAULT_PORT: number;
    static DEFAULT_HOST: string;
    static MTU: number;
    static MSG_FLAGS_VERSION_OFFSET: number;
    static MSG_FLAGS_SRC_OMIT: number;
    static MSG_FLAGS_SRC_PRESENT: number;
    static MSG_FLAGS_DST_OMIT: number;
    static MSG_FLAGS_DST_PRESENT: number;
    static MSG_FLAGS_DST_GROUP: number;
    static SEC_FLAGS_PRIVACY: number;
    static SEC_FLAGS_CONTROL: number;
    static SEC_FLAGS_MX: number;
    static MSG_SESSION_TYPE_UNICAST: number;
    static MSG_SESSION_TYPE_GROUP: number;
    static EXC_FLAGS_INITIATOR: number;
    static EXC_FLAGS_ACK: number;
    static EXC_FLAGS_RELIABILITY: number;
    static EXC_FLAGS_SX: number;
    static EXC_FLAGS_VENDOR: number;
    static NODE_ID_ANY: bigint;
    /**
     * Encodes the given flags into a MessageFlags byte.
     *
     *   |===
     *   |~bit~ ~7~|~6~|~5~|~4~|~3~ | ~2~|~1~|~0~
     * 4+| Version             | -  |S 2+| DSIZ
     *   |===
     *
     * @param {number} dsiz destination size
     * @param {number} sFlag source present flags
     * @param {number} version Version nibble
     * @returns {number} Encoded message flags
     */
    static encodeMessageFlags(dsiz?: number, sFlag?: number, version?: number): number;
    /**
     * Encodes the given flags into a SecurityFlags byte.
     *
     * |===
     * |~bit~ ~7~|~6~|~5~   |~4~|~3~|~2~ |~1~|~0~
     * | P       | C | MX 3+| Reserved 2+| Session Type
     * |===
     *
     * @param {number} sessionType session type nibble
     * @param {number} cFlag control message flag
     * @param {number} pFlag privacy flag
     * @param {number} mxFlag message extensions flag
     *
     * @returns {number} Encoded security flags
     */
    static encodeSecurityFlags(sessionType: number, cFlag?: number, pFlag?: number, mxFlag?: number): number;
    /**
     * Encodes the given flags into an ExchangeFlags byte.
     *
     * |===
     * |~bit~ ~7~|~6~|~5~|~4~|~3~ |~2~|~1~|~0~
     * |   -     | - | - | V | SX | R | A | I
     * |===
     *
     * @param {number} iFlag initiator flag
     * @param {number} aFlag acknowledgement flag
     * @param {number} rFlag reliability flag
     * @param {number} vFlag vendor protocol flag
     * @param {number} sxFlag secure extensions flag
     *
     * @returns {number} Encoded exchange flags
     */
    static encodeExchangeFlags(iFlag: number, aFlag: number, rFlag: number, vFlag?: number, sxFlag?: number): number;
    /**
     * Encodes the given fields into an unencrypted message header.
     *
     * @param {number} msgFlags Message flags byte
     * @param {number} sessionId Session Id uint16
     * @param {number} secFlags Security flags byte
     * @param {number} msgCounter Message counter
     * @param {BigInt} src Source Node Id or null
     * @param {BigInt} dst Destination Node Id or null
     * @param {number} msgLength Message length or null
     * @returns {Buffer} Encoded message header as Buffer
     */
    static encodeHeader(msgFlags: number, secFlags: number, sessionId: number, msgCounter: number, src?: BigInt, dst?: BigInt): Buffer;
    /**
     * Encode the Matter Protocol Header with the given parameters.
     *
     * @param {number} exchangeFlags Exchange Flags field
     * @param {number} opcode Protocol Opcode
     * @param {number} exchangeId Exchange Identifier
     * @param {number} protocolId Protocol Identifier
     * @param {number} vendorId Protocol Vendor Identifier
     * @param {number} ackMsgCounter Acknowledgement Message Counter
     * @returns {Buffer} Encoded protocol header as Buffer
     */
    static encodeProtocolHeader(exchangeFlags: number, exchangeId: number, opcode: number, protocolId: number, vendorId?: number, ackMsgCounter?: number): Buffer;
    /**
     * Extract nonce from given message.
     *
     * @param {Buffer} msg Message to extract Nonce from.
     *
     * @returns {Buffer} Nonce of given message.
     */
    static getNonce(msg: Buffer, srcNode?: bigint): Buffer;
    /**
     * Encrypt the given message with the provided key.
     *
     * @param {Buffer} msg Plaintext message to encrypt.
     * @param {Buffer} key EncryptionKey to use to encrypt message.
     * @param {Buffer} nonce Extract from message in big endian format.
     */
    static encrypt(msg: Buffer, key: Buffer): Buffer;
    /**
     * Decrypt the given message with the provided key.
     *
     * @param {Buffer} enc Encrypted message to decrypt.
     * @param {Buffer} key EncryptionKey to use to decrypt message.
     */
    static decrypt(msg: any, key: Buffer): Buffer | null;
}
