export = PaseSession;
/**
 * PaseSession handles the PAKE protocol to establish a secure session based
 * on a shared passcode.
 *
 * TODO:
 * - [ ] Add timeout for state machine
 * - [ ] Fix state machine to be tolerant to filures
 * - [ ] Properly activate negotiated key for subsequnet session
 */
declare class PaseSession extends EventEmitter {
    static RESPONSE_TIMEOUT: number;
    static PBKDF_ITERATIONS_MIN: number;
    static PBKDF_ITERATIONS_MAX: number;
    static DEFAULT_PINCODE: number;
    static PASE_CONTEXT_SALT: string;
    static State: {
        IDLE: number;
        AWAIT_PBKDF_PARAM_RSP: number;
        AWAIT_PAKE1: number;
        AWAIT_PAKE2: number;
        AWAIT_PAKE3: number;
    };
    static Role: {
        INITIATOR: number;
        RESPONDER: number;
    };
    static create(exchange: any, sessionMgr: any): PaseSession;
    /**
     * PASE PBKDF Request message.
     *
     * pbkdfparamreq-struct => STRUCTURE [ tag-order ]
     * {
     *    initiatorRandom    [1] : OCTET STRING [ length 32 ],
     *    initiatorSessionId [2] : UNSIGNED INTEGER [ range 16-bits ],
     *    passcodeId         [3] : UNSIGNED INTEGER [ length 16-bits ],
     *    hasPBKDFParameters [4] : BOOLEAN,
     *    initiatorMRPParams [5, optional] : mrp-parameter-struct
     * }
     *
     *  mrp-parameter-struct => STRUCTURE [ tag-order ]
     * {
     *    MRP_RETRY_INTERVAL_IDLE    [1, optional] : UNSIGNED INTEGER [ range 16-bits ],
     *    MRP_RETRY_INTERVAL_ACTIVE  [2, optional] : UNSIGNED INTEGER [ range 16-bits ]
     * }
     */
    static TemplateMsgPasePbkdfParamRequest: (params: any) => {
        type: string;
        value: {
            tag: number;
            type: string;
            value: any;
        }[];
    };
    static MapMsgPasePbkdfParamRequest: Map<number, string>;
    /**
     * PASE PBKDF Parameter Response message.
     *
     * pbkdfparamresp-struct => STRUCTURE [ tag-order ]
     * {
     *   initiatorRandom    [1] : OCTET STRING [ length 32 ],
     *   responderRandom    [2] : OCTET STRING [ length 32 ],
     *   responderSessionId [3] : UNSIGNED INTEGER [ range 16-bits ],
     *   pbkdf_parameters   [4] : Crypto_PBKDFParameterSet,
     *   responderMRPParams [5, optional] : mrp-parameter-struct
     * }
     *
     * Crypto_PBKDFParameterSet => STRUCTURE [ tag-order ]
     * {
     *   iterations [1] : UNSIGNED INTEGER [ range 32-bits ],
     *   salt [2] : OCTET STRING [ length 16..32 ],
     * }
     *
     * mrp-parameter-struct => STRUCTURE [ tag-order ]
     * {
     *   MRP_RETRY_INTERVAL_IDLE    [1, optional] : UNSIGNED INTEGER [ range 16-bits ],
     *   MRP_RETRY_INTERVAL_ACTIVE  [2, optional] : UNSIGNED INTEGER [ range 16-bits ]
     * }
     */
    static TemplateMsgPasePbkdfParamResponse: (params: any) => {
        type: string;
        value: {
            tag: number;
            type: string;
            value: any;
        }[];
    };
    static MapMsgPasePbkdfParamResponse: Map<number, string>;
    /**
     * PASE Pake1 Message.
     *
     * pakeDissect-1-struct => STRUCTURE [ tag-order ]
     * {
     *   pA [1] : OCTET STRING [ length CRYPTO_PUBLIC_KEY_SIZE_BYTES ],
     * }
     */
    static TemplateMsgPasePake1: (params: any) => {
        type: string;
        value: {
            tag: number;
            type: string;
            value: any;
        }[];
    };
    /**
     * Dissect PASE Pake2 message.
     *
     * pake-2-struct => STRUCTURE [ tag-order ]
     * {
     *   pB [1] : OCTET STRING [ length CRYPTO_PUBLIC_KEY_SIZE_BYTES ],
     *   cB [2] : OCTET STRING [ length CRYPTO_HASH_LEN_BYTES],
     * }
     */
    static TemplateMsgPasePake2: (params: any) => {
        type: string;
        value: {
            tag: number;
            type: string;
            value: any;
        }[];
    };
    /**
     * Dissect PASE Pake3 message.
     *
     * pake-3-struct => STRUCTURE [ tag-order ]
     * {
     *   cA [1] : OCTET STRING [length CRYPTO_HASH_LEN_BYTES],
     * }
     *
     * sigma-error-enum => UNSIGNED INTEGER [ range 8bits ] {
     *   NoSharedTrustRoots = 0x01,
     *   InvalidParameter = 0x02,
     * }
     */
    static TemplateMsgPasePake3: (params: any) => {
        type: string;
        value: {
            tag: number;
            type: string;
            value: any;
        }[];
    };
    constructor(sessionMgr: any);
    _establishmentExchange: any;
    _state: number;
    _stateTimeout: NodeJS.Timeout | undefined;
    _sessionManager: any;
    _paramPaircode: number;
    _paramIterations: number;
    _paramSalt: string;
    _paramSaltHex: string;
    _localRole: number;
    _initiatorRandom: any;
    _initiatorSessionId: any;
    _passcodeId: any;
    _responderRandom: any;
    _responderSessionId: any;
    _pA: any;
    _pB: any;
    _cA: any;
    _cB: any;
    _pbkdfRequest: any;
    _pbkdfResponse: any;
    _spake2: any;
    _spakeContext: Buffer | undefined;
    _spakeVerifier: any;
    _spakeServer: any;
    _spakeClient: any;
    set Exchange(arg: any);
    get Exchange(): any;
    setTimeout(interval: any): void;
    clearTimeout(): void;
    setState(state: any): void;
    destroy(): void;
    /**
     * Calculate the PAKEPasscodeVerifier stored on device as defined in 11.19.8.1
     * per Crypto_PAKEValues_Responder procedure defined in 3.10.
     */
    generateVerifier(iterations?: undefined, salt?: undefined): Promise<any>;
    /**
     * Compute spake2+ context per spec 3.10.3:
     *
     *   Context := Crypto_Hash("Matter PAKE V1 Commissioning" || PBKDFParamRequest || PBKDFParamResponse)
     */
    computeContext(): Buffer;
    /**
     * Initiate a PASE session as a client to the given host and port.
     * Assumes PaseSession has been initialized with a valid unsecured session and exchange:
     *
     *    var exchange = theNode.ExchangeManager.newExchange()
     *    var pase = theNode.SessionProtocol.newPase(exchange)
     *    pase.startPase()
     *
     * @param {*} host
     * @param {*} port
     * @param {*} passcode
     * @returns
     */
    startPase(host: any, port: any, passcode: any): void;
    /**
     * Send first message of PASE as client.  Assumes PaseSession has been initialized
     * with a valid unsecured session and exchange:
     *
     *    var exchange = theNode.ExchangeManager.newExchange()
     *    var pase = theNode.SessionProtocol.newPase(exchange)
     *    pase.startPase()
     */
    sendPbkdfParamRequest(): void;
    onPbkdfParamRequest(msg: any): void;
    sendPbkdfParamResponse(): void;
    onPbkdfParamResponse(msg: any): Promise<void>;
    sendPake1(): void;
    onPake1(msg: any): Promise<void>;
    sendPake2(): void;
    onPake2(msg: any): Promise<void>;
    sendPake3(): void;
    onPake3(msg: any): void;
    /**
     * Boot-straps the PASE secure session with the derived secret.
     *
     * PASE Session Key KDF
     *
     * byte SEKeys_Info[] = "SessionKeys" =
     *      { 0x53, 0x65, 0x73, 0x73, 0x69, 0x6f, 0x6e, 0x4b, 0x65, 0x79, 0x73 }
     *
     * I2RKey || R2IKey || AttestationChallenge = Crypto_KDF
     * (
     *      inputKey = Ke,
     *      salt = [],
     *      info = SEKeys_Info,
     *      len = 3 * CRYPTO_SYMMETRIC_KEY_LENGTH_BITS
     * )
     *
     * @param {*} secret
     */
    initPaseSession(session: any): void;
    /**
     * Creates the new secure session after PASE key negotiation,
     * and injects the negotiated credentials.
     */
    createSecureSession(): void;
    sendPakeError(): void;
    onPakeError(msg: any): void;
    opcodeName(opcode: any): string;
    onExchangeMessage(msg: any, exchange: any): void;
    #private;
}
import EventEmitter = require("events");
