"use strict";
/**
 * Copyright 2022 Project CHIP Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
const Message = require("../../message/Message");
const SessionProtocolConst = require("./SessionProtocolConst");
class StatusReport {
    /**
     * Send a Status Report with the given payload format:
     *
     * +----------+--------------+------------------------------------------------------
     * | SIZE     | FIELD        | DESCRIPTION
     * | [bytes]  |              |
     * |----------|--------------|------------------------------------------------------
     * |    2     | GeneralCode  |
     * |    4     | ProtocolId   | Protocol-Specific Status
     * |    2     | ProtocolCode | Protocol-Specific Status
     * | Variable | ProtocolData | Optional Protocol-Specific TLV, MAY be empty
     * +----------+--------------+------------------------------------------------------
     *
     * @param {*} exchange Exchange to send status report to
     * @param {*} payload Buffer with preformatted status report message (per spec above)
     */
    static send(exchange, payload) {
        var msg = new Message();
        msg.ProtocolId = StatusReport.PROTOCOL_ID;
        msg.ProtocolOpcode = StatusReport.OPCODE;
        msg.AppPayload = payload;
        exchange.sendMessage(msg);
    }
    static sendStatus(exchange, generalCode, protocolId, protocolCode, protocolData = undefined) {
        var payload = Buffer.alloc(8);
        payload.writeUint16LE(generalCode);
        payload.writeUint32LE(protocolId);
        payload.writeUint16LE(protocolCode);
        if (protocolData != undefined) {
            payload = Buffer.concat([payload, protocolData]);
        }
        StatusReport.send(exchange, payload);
    }
    static sendSuccess(exchange, protocolId, protocolCode, protocolData = undefined) {
        StatusReport.sendStatus(exchange, StatusReport.GeneralCode.SUCCESS, protocolId, protocolCode, protocolData);
    }
}
StatusReport.PROTOCOL_ID = SessionProtocolConst.PROTOCOL_ID;
StatusReport.OPCODE = SessionProtocolConst.Command.StatusReport;
StatusReport.GeneralCode = {
    SUCCESS: 0,
    FAILURE: 1,
    BAD_PRECONDITION: 2,
    OUT_OF_RANGE: 3,
    BAD_REQUEST: 4,
    UNSUPPORTED: 5,
    UNEXPECTED: 6,
    RESOURCE_EXHAUSTED: 7,
    BUSY: 8,
    TIMEOUT: 9,
    CONTINUE: 10,
    ABORTED: 11,
    INVALID_ARGUMENT: 12,
    NOT_FOUND: 13,
    ALREADY_EXISTS: 14,
    PERMISSION_DENIED: 15,
    DATA_LOSS: 16, ///< Unrecoverable data loss or corruption has occurred.
};
module.exports = StatusReport;
//# sourceMappingURL=StatusReport.js.map