/**
 * Copyright 2022 Project CHIP Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/// <reference types="node" />
import { TlvType } from './TlvType';
export declare class TlvCodec {
    static TLV_TYPE_SIGNED_INTEGER: number;
    static TLV_TYPE_UNSIGNED_INTEGER: number;
    static TLV_TYPE_BOOLEAN_FALSE: number;
    static TLV_TYPE_BOOLEAN_TRUE: number;
    static TLV_TYPE_FLOAT: number;
    static TLV_TYPE_DOUBLE: number;
    static TLV_TYPE_UTF8_STRING: number;
    static TLV_TYPE_BYTE_STRING: number;
    static TLV_TYPE_NULL: number;
    static TLV_TYPE_STRUCTURE: number;
    static TLV_TYPE_ARRAY: number;
    static TLV_TYPE_LIST: number;
    static TLV_TYPE_END: number;
    static TLV_TAG_CONTROL_ANONYMOUS: number;
    static TLV_TAG_CONTROL_CONTEXT_SPECIFIC: number;
    static TLV_TAG_CONTROL_COMMON_PROFILE_2Bytes: number;
    static TLV_TAG_CONTROL_COMMON_PROFILE_4Bytes: number;
    static TLV_TAG_CONTROL_IMPLICIT_PROFILE_2Bytes: number;
    static TLV_TAG_CONTROL_IMPLICIT_PROFILE_4Bytes: number;
    static TLV_TAG_CONTROL_FULLY_QUALIFIED_6Bytes: number;
    static TLV_TAG_CONTROL_FULLY_QUALIFIED_8Bytes: number;
    static INT8_MIN: number;
    static INT16_MIN: number;
    static INT32_MIN: number;
    static INT64_MIN: number;
    static INT8_MAX: number;
    static INT16_MAX: number;
    static INT32_MAX: number;
    static INT64_MAX: number;
    static UINT8_MAX: number;
    static UINT16_MAX: number;
    static UINT32_MAX: number;
    static UINT64_MAX: number;
    static ElementTypes: Record<number, string>;
    static TagControls: Record<number, string>;
    /**
     * Encode a value as a TLV signed integer.
     *
     * @param {number|BigInt} val
     */
    static encodeSignedInt(val: number | bigint): Buffer;
    static encodeU64(val: number | bigint): Buffer;
    /**
     * Encode a value as a TLV unsigned integer.
     *
     * @param {number} val
     */
    static encodeUnsignedInt(val: number): Buffer;
    /**
     * Encode a value as a TLV floating point.
     *
     * @param {number} val
     */
    static encodeFloat(val: number): Buffer;
    /**
     * Encode a value as a TLV double.
     *
     * @param {number} val
     */
    static encodeDouble(val: number): Buffer;
    static encodeTag(tag: number): Buffer;
    /**
     * Encode the TLV Control Octet containing Tag Control and Element Type fields.
     *
     * @param {number} type
     * @param {number} lenOfLenOrVal
     * @param {number?} tag
     * @param {number?} profile
     */
    static encodeControlByte(type: number, lenOfLenOrVal?: number, tag?: number, profile?: number): Buffer;
    /**
     * Encode the TLV Control Octet containing Tag Control and Element Type fields.
     *
     * @param {Buffer} tlv
     */
    static decodeControlByte(tlv: Buffer): TlvType.DecodedControlByte;
    /**
     * Decodes the first byte of a TLV element into an element {value, length} tuple.
     * @param {Buffer} tlv
     * @param {number} elementType
     * @returns { TlvType.ValueLength}
     */
    static decodeValueAndLength(tlv: Buffer, elementType: number): TlvType.ValueLength;
    /**
     * The control byte specifies the type of a TLV element and how its tag, length and value
     * fields are encoded. The control byte consists of two subfields:
     * an element type field which occupies the lower 5 bits,
     * and a tag control field which occupies the upper 3 bits. The element type field encodes the element’s type
     * as well as how the corresponding length and value fields are encoded.  In the case of Booleans and the
     * null value, the element type field also encodes the value itself."
     *
     * @param {Buffer} tlv
     * @param {number} tagType
     * @returns {Object} deterministic json representation of tlv
     */
    static decodeTagAndLength(tlv: Buffer, tagType: number): TlvType.DecodedTagLength;
}
