/**
 * Copyright 2022 Project CHIP Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/// <reference types="node" />
import { TlvType } from './TlvType';
export declare class TlvWriter {
    buf: Buffer;
    constructor(buf?: undefined);
    get Buffer(): Buffer;
    /**
     * Write a value as a TLV unsigned integer with the specified TLV tag.
     *
     * @param {number?} tag
     * @param {number} val An unsigned integer number to encode as TLV value
     */
    putUnsignedInt(val: number, tag?: number, profile?: number): void;
    /**
     * Write a value as a TLV signed integer with the specified TLV tag.
     *
     * @param {*} tag
     * @param {number} val A signed integer number to encode as TLV value
     */
    putSignedInt(val: number, tag?: number, profile?: number): void;
    /**
     * Write a value as a TLV float with the specified TLV tag.
     *
     * @param {*} tag
     * @param {number} val A floating point number to encode as TLV value
     */
    putFloat(val: number, tag?: number, profile?: number): Buffer;
    /**
     * Write a value as a TLV double with the specified TLV tag.
     *
     * @param {*} tag
     * @param {number} val A floating point number to encode as TLV value
     */
    putDouble(val: number, tag?: number, profile?: number): void;
    /**
     * Write a value as a TLV boolean with the specified TLV tag.
     *
     * @param {*} tag
     * @param {boolean} val A boolean to encode as TLV value
     */
    putBoolean(val: boolean, tag?: number, profile?: number): void;
    /**
     * Write a TLV null with the specified TLV tag.
     *
     * @param {*} tag
     */
    putNull(tag?: number, profile?: number): void;
    /**
     * Write a value as a TLV byte string with the specified TLV tag.
     *
     * @param {*} tag
     * @param {*} val A Buffer to encode as TLV value
     */
    putBytes(val: Buffer, tag?: number, profile?: number): void;
    /**
     * Write a value as a TLV utf-8 string with the specified TLV tag.
     *
     * @param {*} tag
     * @param {*} val A string to encode as TLV value
     */
    putString(val: string, tag?: number, profile?: number): void;
    putStructStart(tag?: number, profile?: number): Buffer;
    putArrayStart(tag?: number, profile?: number): Buffer;
    putListStart(tag?: number, profile?: number): Buffer;
    putContainerEnd(): void;
    /**
     * Write a generic value as the appropriate TLV encoding with the specified TLV tag.
     *
     * @param {*} tag
     * @param {*} val A value of any type (int, float, string, bool, null)
     */
    put(val: any, tag?: number, profile?: number): void;
    /**
     * Encodes the given json into binary tlv format.
     *
     * Json encoded as follows:
     * {
     *  "profile": number or undefined=none
     *  "tag": number or undefined=anonymous, 0=common, -1=implicit
     *  "type": string from TlvCodec.ElementTypes or undefined for auto-detect
     *  "value": raw value or [] for containers.  undefined for null
     * }
     *
     * See theTestTlvJsonVector in TestTlvJson.js for examples.
     *
     * @param {*} json JSON to encode
     */
    encode(json: TlvType.DeterministicJson): this;
}
